from random import random
import math

def num_weights(num_inputs, num_hidden, num_outputs):
    """ Return number of weights needed """
    return num_hidden * num_inputs + num_outputs * num_hidden

class Neuron:
    def __init__(self, num_inputs, weights):
        assert len(weights) == num_inputs
        self.num_inputs = num_inputs
        self.weights = weights
    
    def transfer_func(self, x):
        """ Sigmoid function in output range [-1.0, 1.0] """
        #return 1.0 / (1.0 + math.exp(-x))
        return 2.0 * (1.0 / (1.0 + math.exp(x))) - 1.0

    def activate(self, inputs):
        assert len(inputs) == self.num_inputs
        sum = 0.0
        for input, weight in zip(inputs, self.weights):
            sum += weight * input
        self.output = self.transfer_func(sum)
        #print "inputs:", inputs, "\nWeights:", self.weights, "\noutput:", self.output

class Brain:
    def __init__(self, num_inputs, num_hidden, num_outputs, all_weights=None):
        num_all_weights = num_weights(num_inputs, num_hidden, num_outputs)
        if all_weights == None:
            all_weights = [2.0 * random() - 1.0 for i in xrange(num_all_weights)]
        # weights: hidden * inputs, outputs * hidden
        #print "Needed: ", num_all_weights, " Got:", len(all_weights)
        assert len(all_weights) == num_all_weights
        self.all_weights = all_weights
        self.num_inputs = num_inputs
        self.num_hidden = num_hidden
        self.num_outputs = num_outputs
        self.input_neurons = []
        self.hidden_neurons = []
        self.output_neurons = []
        
        #print "Input layer"
        for i in xrange(num_inputs):
            self.input_neurons.append(Neuron(1, [1.0]))
        #print "Hidden layer"
        startw = 0
        for i in xrange(num_hidden):
            weights = all_weights[startw:startw+num_inputs]
            all_weigths = all_weights[num_inputs:]
            self.hidden_neurons.append(Neuron(num_inputs, weights))
            startw += num_inputs
        #print "Output layer"
        for i in xrange(num_outputs):
            weights = all_weights[startw:startw+num_hidden]
            all_weigths = all_weights[num_hidden:]
            self.output_neurons.append(Neuron(num_hidden, weights))
            startw += num_hidden
    
    def activate(self, input_values):
        """ Return output of brain """
        assert len(input_values) == self.num_inputs
        
        # Activate input neurons
        for value, neuron in zip(input_values, self.input_neurons):
            neuron.activate([value])
        input_values = [neuron.output for neuron in self.input_neurons]
        
        # Activate hidden neurons
        for neuron in self.hidden_neurons:
            neuron.activate(input_values)
        input_values = [neuron.output for neuron in self.hidden_neurons]
        
        # Activate output neurons
        for neuron in self.output_neurons:
            neuron.activate(input_values)
        output_values = [neuron.output for neuron in self.output_neurons]
        
        return output_values
    
    # def get_all_weights(self):
        # return self.all_weights
        