import wx, math
from random import random
from dna import Dna

ADOLESCENT_AGE = 200
ACCENT_DURATION = 15
PREGNANT_DURATION = 120

def mix_colors(color1, color2):
    #return wx.Color([int(round((val1 + val2) / 2.0)) for (val1, val2) in zip(color1, color2)])
    r = int(round((color1[0] + color2[0]) / 2.0))
    g = int(round((color1[1] + color2[1]) / 2.0))
    b = int(round((color1[2] + color2[2]) / 2.0))
    return (r, g, b)
    
def values_to_color(values):
    """ floats: tuple with three values 0..1 """
    colors = []
    for value in values:
        c = int(256.0 * value)
        if c < 0:
            c = 0
        if c > 255:
            c = 255
        colors.append(c)
    return wx.Color(colors[0], colors[1], colors[2])
    
class Bug:
    def __init__(self, operator, dna=None):
        self.operator = operator

        # Initial values, to be updated soon
        self.close_bug = None
        self.close_grass = None
        self.pregnant = False
        self.pregnant_remaining = 0
        self.embryos = []
    
        # Fixed values for every bug
        self.num_embryos = 1 + int(4 * random())
        self.muscle_funcs = [self.forward, self.turn, self.eat, self.mate]
        self.num_brain_inputs = 7
        #self.num_brain_hiddens = 2
        self.num_brain_outputs = len(self.muscle_funcs)
        self.accent = 0
        self.age = 0 #ADOLESCENT_AGE
        self.energy = 1.0
        self.birth_size = .01
        self.max_size = .03
        self.max_speed = 0.03
        self.max_turn = 0.2 * math.pi
        self.eye_size = .007
        #self.eye_color = wx.Color(0, 0, 255)
        self.eye_range = 0.25
        self.eye_angle_range = 0.4 * math.pi
        
        # Random values for every bug
        self.x = random()
        self.y = random()
        self.angle = 2.0 * math.pi * random()

        # Set dna
        if dna == None:
            dna = Dna(num_brain_inputs=self.num_brain_inputs,
                      #num_brain_hiddens=self.num_brain_hiddens,
                      num_brain_outputs=self.num_brain_outputs)
        self.dna = dna

        # Properties derived from dna
        self.brain = self.dna.get_brain()
        self.color = self.dna.get_color()
        self.eye_color = self.dna.get_eye_color()
        self.eye_angle = self.dna.get_eye_angle() #0.25 * math.pi

        # Temporary properties
        self.size = self.max_size # will be recalculated in tick()
        self.vision_colors = [None, None] # will be recalculated in tick()
    
    def get_dna(self):
        return self.dna
        
    def get_vision_colors(self):
        return self.vision_colors
        
    def conception(self, donor_dna):
        """ Got dna from another bug's ejaculation """
        own_dna = self.get_dna()
        if not self.pregnant and self.age >= ADOLESCENT_AGE:
            for i in xrange(self.num_embryos):
                dna = Dna(donor_dna, own_dna)
                #embryo_dna = mutate(crossover(own_dna, donor_dna))
                embryo = Bug(self.operator, dna)
                embryo.accent = ACCENT_DURATION
                embryo.age = 0
                self.embryos.append(embryo)
            self.pregnant = True
            self.pregnant_remaining = PREGNANT_DURATION
            self.accent = ACCENT_DURATION
            self.operator.sound("tlok.wav")
            print "Got pregnant!"
        #num_embryos = 2

    def give_birth(self):
        assert self.pregnant
        assert len(self.embryos) > 0
        while self.embryos:
            kid = self.embryos.pop()
            kid.x = self.x + (random() - 0.5) / 15.0
            kid.y = self.y + (random() - 0.5) / 15.0
            kid.energy = self.energy - 0.1 #(0.5 + self.energy) / 2.0
            #kid.energy = (0.4 + self.energy) / 2.0
            #kid.energy = 0.1 + .9 * self.energy
            self.operator.add_bug(kid)
            self.operator.sound("jippie.wav")
            print "A kid is born!"
            self.tire(0.06)
        self.pregnant = False
    
    def draw_sight(self, dc, x, y, angle):
        scale = min(dc.GetSize())
        angle_range = self.eye_angle_range
        range = self.eye_range
        x1, y1 = scale * x, scale * y
        x2 = scale * (x + range * math.cos(angle - angle_range / 2.0))
        y2 = scale * (y + range * math.sin(angle - angle_range / 2.0))
        dc.DrawLine(x1, y1, x2, y2)

        x1, y1 = scale * x, scale * y
        x2 = scale * (x + range * math.cos(angle + angle_range / 2.0))
        y2 = scale * (y + range * math.sin(angle + angle_range / 2.0))
        dc.DrawLine(x1, y1, x2, y2)

        x1, y1 = scale * x, scale * y
        x2 = scale * (x + range / 2 * math.cos(angle))
        y2 = scale * (y + range / 2 * math.sin(angle))
        dc.DrawLine(x1, y1, x2, y2)
        
    def draw(self, dc, scale):
        
        
        # Draw sight
        #self.draw_sight(dc, self.x, self.y, self.angle - self.eye_angle / 2)
        #self.draw_sight(dc, self.x, self.y, self.angle + self.eye_angle / 2)
        
        # Draw body
        x = self.x * scale
        y = self.y * scale
        radius = self.size * scale
        color = self.color
        dc.SetBrush(wx.Brush(wx.Color(color[0], color[1], color[2])))
        if self.accent:
            # make thick body to draw attention
            if self.accent % 2 == 0:
                dc.SetPen(wx.Pen(wx.BLACK, width=3))
            else:
                dc.SetPen(wx.Pen(wx.WHITE, width=3))
            self.accent -= 1
        dc.DrawCircle(x, y, radius)
        dc.SetPen(wx.Pen(wx.BLACK, width=1))
        
        # Draw eyes
        eye_radius = self.eye_size * scale
        eye1_x = x + radius * math.cos(self.angle - self.eye_angle / 2)
        eye1_y = y + radius * math.sin(self.angle - self.eye_angle / 2)
        eye2_x = x + radius * math.cos(self.angle + self.eye_angle / 2)
        eye2_y = y + radius * math.sin(self.angle + self.eye_angle / 2)
        color = self.eye_color
        dc.SetBrush(wx.Brush(wx.Color(color[0], color[1], color[2])))
        dc.DrawCircle(eye1_x, eye1_y, eye_radius)
        dc.DrawCircle(eye2_x, eye2_y, eye_radius)
        
        # Draw embryos
        if self.pregnant:
            angle = self.angle
            embryo_radius = scale * self.birth_size * (1.0 - float(self.pregnant_remaining) / PREGNANT_DURATION)
            for embryo in self.embryos:
                embryo_x = (self.x + .35 * self.size * math.cos(angle)) * scale
                embryo_y = (self.y + .35 * self.size * math.sin(angle)) * scale
                color = embryo.color
                dc.SetBrush(wx.Brush(wx.Color(color[0], color[1], color[2])))
                dc.DrawCircle(embryo_x, embryo_y, embryo_radius)
                angle += 2.0 * math.pi / len(self.embryos)
        
        # Draw energy bar border
        borleft = x - 10
        bortop = y + radius + 3
        borw = 21
        borh = 5
        dc.SetBrush(wx.Brush(wx.Color(255, 255, 255)))
        dc.DrawRectangle(borleft, bortop, borw, borh)
        
        # Draw energy bar
        barleft = borleft
        bartop = bortop
        barw = round(self.energy * borw)
        barh = borh
        dc.SetBrush(wx.Brush(wx.Color(0, 255, 0)))
        dc.DrawRectangle(barleft, bartop, barw, barh)
    
    # def set_inputs(self, inputs):
        # pass
    
    def tire(self, amount):
        self.energy -= amount
        if self.energy <= 0.0:
            self.energy = 0.0
            #self.die()
    
    def heal(self, amount):
        self.energy += amount
        if self.energy > 1.0:
            self.energy = 1.0
            
    # def die(self):
        # pass
    
    def forward(self, amount):
        #print "Forward %5.1f" % amount
        if amount > 1.0:
            amount = 1.0
        if amount < -0.4:
            amount = -0.4
        speed = amount * self.max_speed
        self.x += speed * math.cos(self.angle)
        self.y += speed * math.sin(self.angle)
        if self.x < 0.0:
            self.x = 0.0
        if self.x > 1.0:
            self.x = 1.0
        if self.y < 0.0:
            self.y = 0.0
        if self.y > 1.0:
            self.y = 1.0
        self.tire(.004 * amount ** 2)
    
    def turn(self, amount):
        #print "Turn %5.1f" % amount
        if amount < -1.0:
            amount = -1.0
        if amount > 1.0:
            amount = 1.0
        self.angle += amount * self.max_turn
        if self.angle < 0.0:
            self.angle += 2.0 * math.pi
        if self.angle >= 2.0 * math.pi:
            self.angle -= 2.0 * math.pi
        self.tire(.0005 * amount)
    
    def eat(self, amount):
        #print "Eat", amount
        if amount < 0.0:
            amount = 0.0
        if amount > 1.0:
            amount = 1.0
        if self.close_grass:
            food_amount = self.close_grass.shrink(amount)
            if food_amount > 0.0:
                self.operator.sound("eat2.wav")
                self.heal(food_amount)
                #print "Eating."
                self.accent = ACCENT_DURATION
        self.tire(.0011)
    
    def mate(self, amount):
        if amount > 0.0 and self.close_bug and self.age >= ADOLESCENT_AGE:
            self.accent = ACCENT_DURATION
            self.close_bug.conception(self.dna)
        else:
            self.tire(.003)

    def tick(self, operator):
        """ Got a clock tick from the operator; do my thing """
        # Offspring things
        if self.pregnant:
            self.pregnant_remaining -= 1
            if self.pregnant_remaining == 0:
                self.give_birth()
        
        # Size calculation
        self.size = self.age / 20000.0 + .01
        if self.size > self.max_size:
            self.size = self.max_size
        
        # Vision
        l_color, (l_bug, l_bug_dist), (l_grass, l_grass_dist) = \
                 operator.get_sight(self, self.angle - self.eye_angle / 2)
        r_color, (r_bug, r_bug_dist), (r_grass, r_grass_dist) = \
                 operator.get_sight(self, self.angle + self.eye_angle / 2)
        
        self.vision_colors = [values_to_color(l_color), values_to_color(r_color)]

        # Secret knowledge about close objects
        self.close_bug = None
        if l_bug_dist < r_bug_dist:
            if l_bug and (l_bug_dist <= (l_bug.size + self.size) / 2.0):
                self.close_bug = l_bug
        else:
            if r_bug and (r_bug_dist <= (r_bug.size + self.size) / 2.0):
                self.close_bug = r_bug
            
        # if (l_bug or r_bug) and (l_bug == r_bug) \
           # and min(l_bug_dist, r_bug_dist) < 1.1 * self.size:
            # self.close_bug = l_bug
            #print "Bug collision"
        # else:
            # self.close_bug = None
        # if (l_grass or r_grass) and (l_grass == r_grass) \
           # and min(l_grass_dist, r_grass_dist) < 1.4 * self.size:
            # self.close_grass = l_grass
            #print "Grass oppurtunity"
        # else:
            # self.close_grass = None

        self.close_grass = None
        if l_grass_dist < r_grass_dist:
            if l_grass and (l_grass_dist <= (l_grass.size + self.size) / 2.0):
                self.close_grass = l_grass
        else:
            if r_grass and (r_grass_dist <= (r_grass.size + self.size) / 2.0):
                self.close_grass = r_grass
            
        # Send stimuli to brains and get brain control
        #brain_inputs = [1.0, self.energy,
        brain_inputs = [l_color[0], l_color[1], l_color[2],
                        r_color[0], r_color[1], r_color[2], self.energy]
        # brain_inputs = [l_color[0] - r_color[0],
                        # l_color[1] - r_color[1],
                        # l_color[2] - r_color[2],
                        # self.energy]
        brain_outputs = self.brain.activate(brain_inputs)
        for output, muscle_func in zip(brain_outputs, self.muscle_funcs):
            muscle_func(output)
        
        # Unescapable forces
        self.tire(0.0001)
        self.age += 1
