import wx

class Graph:
    def __init__(self, color=None, minval=None, maxval=None, num_x=None):
        self.values = []
        if color == None:
            color = wx.BLACK
        self.color = color
        self.minval = minval
        self.maxval = maxval
        self.num_x = num_x
        
    def update(self, value):
        self.values.append(value)
        if len(self.values) > self.num_x:
            self.values = self.values[-self.num_x:]
        
    def draw(self, dc):
        if len(self.values) < 2:
            return
        w, h = dc.GetSize()
        minval = self.minval
        if minval == None:
            minval = min(self.values)
        maxval = self.maxval
        if maxval == None:
            maxval = max(self.values)
        val_range = (maxval - minval)
        if val_range == 0.0:
            val_range = 1.0
        num_x = self.num_x
        if num_x == None:
            num_x = len(self.values)
        x_scale = float(w) / num_x
        y_scale = -float(h) / val_range
        y_offset = h
        dc.SetPen(wx.Pen(self.color))
        x = 0.0
        points = []
        for value in self.values:
            y = (value - minval) * y_scale + y_offset
            points.append(wx.Point(x, y))
            x += x_scale
        dc.DrawLines(points)

class GraphPanel(wx.Panel):
    def __init__(self, parent, colors, ranges, num_x):
        """
        colors: list of wx.Color values, one for each graph
        ranges: list of (min, max) for each graph, possibly (None, None)
        num_x: max data points in graph, possibly None
        """
        wx.Panel.__init__(self, parent) #, style=wx.BORDER_SUNKEN)
        self.num = len(colors)
        self.graphs = []
        for color, (minval, maxval) in zip(colors, ranges):
            self.graphs.append(Graph(color=color, num_x=num_x,
                                     minval=minval, maxval=maxval))

        #self.graphs = [Graph(color=color, num_x=num_x) for color in colors]
        self.Bind(wx.EVT_PAINT, self.on_paint)
    
    # def set_ranges(self, ranges):
        # """ (min, max) pairs for each graph """
        # self.ranges = ranges
    
    def update(self, values):
        for graph, value in zip(self.graphs, values):
            graph.update(value)
            
    def draw(self, dc=None):
        if dc == None:
            cdc = wx.ClientDC(self)
            dc = wx.BufferedDC(cdc)
        dc.Clear()
        for graph in self.graphs:
            graph.draw(dc)
                
    def on_paint(self, event):
        dc = wx.PaintDC(self)
        self.draw(dc)
