from bug import Bug
from grass import Grass
from random import random
from sound import Sound
import window
import math

# Most expensive functions: get_sight, atan2, angle_between

NUM_GRASS = 70
MAX_GRASS = 80
NUM_BUGS = 50
MAX_BUGS = 150

def angle_between(angle, min_angle, max_angle):
    """ Test if angle is between min_angle and max_angle """
    angle = angle % (2.0 * math.pi)
    min_angle = min_angle % (2.0 * math.pi)
    max_angle = max_angle % (2.0 * math.pi)
    # all angles are positive now
    if min_angle > max_angle: # pie part includes positive x axis
        return angle > min_angle or angle < max_angle
    else:
        return min_angle < angle < max_angle

class BugOperator:
    def __init__(self, num_bugs, num_grass):
        #self.max_grass = 70
        #self.max_bugs = 120
        self.favorite_bug = None
        self.bugs = []
        self.grasss = []
        self.injecting_bugs = True
        #for i in xrange(num_bugs):
            #bug = Bug(self, dna=None)
            #self.bugs.append(bug)
        #    self.add_bug()
        for i in xrange(num_grass):
            grass = Grass(self, parent=None)
            grass.amount = .2 + 0.5 * random()
            self.grasss.append(grass)
        self.player = Sound()
    
    def get_bugs(self):
        return self.bugs
    
    def get_grasss(self):
        return self.grasss
        
    def hit_test(self, x, y):
        for bug in self.bugs:
            dist = math.sqrt((x - bug.x) ** 2 + (y - bug.y) ** 2)
            if dist <= bug.size:
                return bug
        return None
        
    def get_sight(self, from_bug, sight_angle):
        """
        Return (color, (closest_bug, dist), (closest_grass, dist))
        """
        #self.paint_sight(x, y, angle, angle_range, range)
        x = from_bug.x
        y = from_bug.y
        angle_range = from_bug.eye_angle_range
        min_angle = sight_angle - angle_range / 2.0
        max_angle = sight_angle + angle_range / 2.0
        
        color = [0.0, 0.0, 0.0]
        closest_bug = None
        closest_bug_dist = 10
        for bug in self.bugs:
            if bug == from_bug:
                continue
            dx = bug.x - x
            dy = bug.y - y
            angle = math.atan2(dy, dx) # -pi..pi. Very slow function!
            if angle < 0.0:
                angle += 2.0 * math.pi # 0..2pi
            if angle_between(angle, min_angle, max_angle):
                # Other bug is within sight angles
                dist = math.sqrt(dx ** 2 + dy ** 2)
                if dist < closest_bug_dist:
                    closest_bug_dist = dist
                    closest_bug = bug
                if dist <= from_bug.eye_range:
                    #print "Bug nearby!"
                    #contrib = 1.0 / (15.0 * dist + 1.0)
                    contrib = (from_bug.eye_range - dist) / from_bug.eye_range
                    color[0] += contrib * bug.color[0] / 255.0
                    color[1] += contrib * bug.color[1] / 255.0
                    color[2] += contrib * bug.color[2] / 255.0
        
        closest_grass = None
        closest_grass_dist = 10
        for grass in self.grasss:
            dx = grass.x - x
            dy = grass.y - y
            angle = math.atan2(dy, dx)
            if angle < 0.0:
                angle += 2.0 * math.pi
            if angle_between(angle, min_angle, max_angle):
                #print "  Food in view"
                dist = math.sqrt(dx ** 2 + dy ** 2)
                if dist < closest_grass_dist:
                    closest_grass_dist = dist
                    closest_grass = grass
                if dist < from_bug.eye_range:
                    #print "Food nearby!"
                    #contrib = 1.0 / (15.0 * dist + 1.0)
                    contrib = (from_bug.eye_range - dist) / from_bug.eye_range
                    color[0] += contrib * grass.amount * grass.color[0] / 255.0
                    color[1] += contrib * grass.amount * grass.color[1] / 255.0
                    color[2] += contrib * grass.amount * grass.color[2] / 255.0
        maxc = max(color)
        if maxc > 1.0:
            color = (color[0] / maxc, color[1] / maxc, color[2] / maxc)
        else:
            color = tuple(color)
        
        return (color, (closest_bug, closest_bug_dist),
                       (closest_grass, closest_grass_dist))
                       
    def get_num_bugs(self):
        return len(self.bugs)
    
    def get_grass_amount(self):
        """ Return the total amount of grass present """
        return sum([grass.amount for grass in self.grasss])
    
    def reproduce_grass(self, parent):
        if len(self.grasss) < MAX_GRASS:
            grass = Grass(self, parent)
            self.grasss.append(grass)
        #else:
        #    grass = None
        #return grass
        
    def reproduce_bug(self, parent_bug1, parent_bug2):
        """ Return True on success """
        if len(self.bugs) < MAX_BUGS:
            bug = Bug(self, parent_bug1, parent_bug2)
            self.bugs.append(bug)
            return True
        else:
            return False
    
    def add_bug(self, bug=None):
        if len(self.bugs) >= MAX_BUGS:
            print "Sorry, bug not added: too many bugs."
        else:
            if bug == None:
                bug = Bug(self, dna=None)
            self.bugs.append(bug)
            if self.favorite_bug == None:
                self.favorite_bug = bug
        print len(self.bugs), "krippers."

    def inject_bug(self):
        if len(self.bugs) >= NUM_BUGS:
            self.injecting_bugs = False
            print "Injection of bugs complete."
        else:
            self.add_bug()
        
    def die(self, bug):
        self.bugs.remove(bug)
        print len(self.bugs), "krippers left."
        if len(self.bugs) <= 1:
            print "Creating new generation..."
            #for i in xrange(NUM_BUGS):
            #    self.add_bug()
            self.injecting_bugs = True
            #print "New generation!"
    
    def tick(self):
        if self.injecting_bugs:
            self.inject_bug()
        for bug in self.bugs:
            bug.tick(self)
            if bug.energy <= 0.0:
                self.die(bug)
        for grass in self.grasss:
            if grass.amount <= 0.0:
                self.grasss.remove(grass)
                print "Grass depleted."
            grass.grow()
        if random() > .925:
            self.reproduce_grass(parent=None)
        
    
    def sound(self, filename):
        #self.player.play(filename)
        pass

if __name__ == "__main__":
    operator = BugOperator(num_bugs=NUM_BUGS, num_grass=NUM_GRASS)
    window.start(operator)
