import sys, time

class Sound:
    def __init__(self):
        self.enabled = False
        platform = sys.platform
        if platform.startswith('win'):
            self.init_windows()
        elif platform.find('linux') > -1:
            self.init_linux()
        else:
            print "Note: sound is disabled on this platform (%s)." % platform
        self.last_sound_time = 0
        
    def init_windows(self):
        from winsound import PlaySound, SND_FILENAME, SND_ASYNC
        self.WinPlaySoundFunc = PlaySound
        self.parameters = SND_FILENAME|SND_ASYNC
        self.play_func = self.play_windows
        self.enabled = True
    
    def play_windows(self, filename):
        self.WinPlaySoundFunc(filename, self.parameters)
        
    def init_linux(self):
        from wave import open as waveOpen
        from ossaudiodev import open as ossOpen
        self.play_func = self.play_linux
        self.enabled = True
        self.LinuxWaveOpen = waveOpen
        self.ossOpen = ossOpen

    def play_linux(self, filename):
        s = self.LinuxWaveOpen(filename, 'rb')
        (nc,sw,fr,nf,comptype, compname) = s.getparams()
        dsp = self.ossOpen('/dev/dsp','w')
        try:
            from ossaudiodev import AFMT_S16_NE
        except ImportError:
            if byteorder == "little":
                AFMT_S16_NE = ossaudiodev.AFMT_S16_LE
            else:
                AFMT_S16_NE = ossaudiodev.AFMT_S16_BE
        dsp.setparameters(AFMT_S16_NE, nc, fr)
        data = s.readframes(nf)
        s.close()
        dsp.write(data)
        dsp.close()

    def play(self, filename):
        #return
        #if time.time() - self.last_sound_time > 3:
        self.last_sound_time = time.time()
        self.play_func(filename)
        
    def get_enabled(self):
        return self.enabled
