import wx
from graphpanel import GraphPanel
from eyepanel import EyePanel

class MainFrame(wx.Frame):
    def __init__(self, parent, id, operator):
        wx.Frame.__init__(self, parent, id, title="Krippers")
        self.operator = operator
        
        # Left part of the screen
        
        self.canvas = wx.Panel(self)
        # (self.canvas_buffer is initialized in on_size)
        self.start_button = wx.Button(self, wx.ID_ANY, "Start")
        self.add_bug_button = wx.Button(self, wx.ID_ANY, "Add 5 krippers")
        
        # (buttons)
        butsiz = wx.BoxSizer(wx.HORIZONTAL)
        butsiz.Add(self.start_button, 0, wx.CENTER)
        butsiz.Add(self.add_bug_button, 0, wx.CENTER)
        wx.EVT_BUTTON(self, self.start_button.GetId(), self.on_button_start)
        wx.EVT_BUTTON(self, self.add_bug_button.GetId(), self.on_add_bug)
        self.canvas.Bind(wx.EVT_LEFT_DOWN, self.on_bug_click)
        
        leftsiz = wx.BoxSizer(wx.VERTICAL)
        leftsiz.Add(self.canvas, 1, wx.EXPAND)
        #siz.Add(self.start_button, 0, wx.CENTER)
        leftsiz.Add(butsiz, 0, wx.CENTER)

        # Right part of the screen
        
        self.singlebug_view = wx.Panel(self, size=(100, 100), style=wx.BORDER_SUNKEN)
        self.graphpanel = GraphPanel(self, [wx.RED, wx.GREEN], [(0, 150), (0, 80)], num_x=1500)
        self.graphpanel.SetSize((100,80))
        self.eyepanel = EyePanel(self)
        
        rightsiz = wx.BoxSizer(wx.VERTICAL)
        rightsiz.Add(self.singlebug_view, 0, wx.TOP)
        rightsiz.Add(self.eyepanel, 0, wx.EXPAND)
        rightsiz.Add(self.graphpanel)
        
        # Main sizer
        
        mainsiz = wx.BoxSizer(wx.HORIZONTAL)
        mainsiz.Add(leftsiz, 1, wx.EXPAND)
        mainsiz.Add(rightsiz, 0, wx.EXPAND)
        
        self.SetSizer(mainsiz)
        self.timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.on_timer)
        self.Bind(wx.EVT_SIZE, self.on_size)
        
        self.paint_count = 0 # used for skipping draw cycles

        #wx.EVT_PAINT(self, self.on_paint)
        #self.Bind(wx.EVT_PAINT, self.on_paint)
        
    def on_timer(self, event):
        self.operator.tick()
        self.draw()
        
    def on_button_start(self, event):
        self.ProcessEvent(wx.SizeEvent(self.GetSize()))
        self.timer.Start(milliseconds = 50)
        
    def on_add_bug(self, event):
        for i in xrange(5):
            self.operator.add_bug()
    
    def on_bug_click(self, event):
        x, y = event.GetPosition()
        scale = min(self.canvas.GetClientSize())
        bug = self.operator.hit_test(float(x) / scale, float(y) / scale)
        if bug != None:
            self.operator.favorite_bug = bug
        
    def on_size(self, event):
        #print "Resizing."
        canvas_w, canvas_h = self.canvas.GetSize()
        self.canvas_buffer = wx.EmptyBitmap(canvas_w, canvas_h)
        event.Skip()

    def draw(self):
        self.draw_grass_and_bugs()
        # Favorite bug
        if self.paint_count % 8 == 0:
            self.draw_favorite_bug()
        # Graphs
        if self.paint_count % 20 == 0:
            self.draw_graphs()
        self.draw_favorite_sight()
        self.paint_count = (self.paint_count + 1) % 100

    def draw_grass_and_bugs(self):
        cdc = wx.ClientDC(self.canvas)
        dc = wx.BufferedDC(cdc, self.canvas_buffer)
        dc.Clear()
        scale = min(dc.GetSize())
        # Grass
        grasss = self.operator.get_grasss()
        for grass in grasss:
            grass.draw(dc, scale)
        # Bugs
        bugs = self.operator.get_bugs()
        for bug in bugs:
            bug.draw(dc, scale)
    
    def draw_favorite_bug(self):
        bug = self.operator.favorite_bug
        if bug != None:
            dc = wx.ClientDC(self.singlebug_view)
            dc.Clear()
            sbv_w, sbv_h = dc.GetSize()
            scale = min(self.canvas.GetClientSize())
            dc.SetDeviceOrigin(-bug.x * scale + sbv_w / 2, -bug.y * scale + sbv_h / 2)
            bug.draw(dc, scale)
    
    def draw_graphs(self):
        num_bugs = self.operator.get_num_bugs()
        grass_amount = self.operator.get_grass_amount()
        self.graphpanel.update([num_bugs, grass_amount])
        self.graphpanel.draw()
        
    def draw_favorite_sight(self):
        bug = self.operator.favorite_bug
        if bug != None:
            colors = bug.get_vision_colors()
            self.eyepanel.update(colors)
    
def start(operator):
    app = wx.PySimpleApp()
    frame = MainFrame(None, wx.ID_ANY, operator)
    frame.Show(True)
    app.MainLoop()
 
