import brain
import math
from random import random

def pick_one_parent(string1, string2):
    out_string = []
    if random() < 0.5:
        in_string = string1
    else:
        in_string = string2
    for elem in in_string:
        if random() < .05:
            out_string.append(2.0 * random() - 1.0)
        elif random() < .1:
            out_string.append(elem + (random() - 0.5) / 4.0)
        else:
            out_string.append(elem)
    return out_string

def crossover(string1, string2):
    assert len(string1) == len(string2)
    out_string = []
    crossover_point = int(random() * len(string1))
    for elem1, elem2 in zip(string1, string2):
        #rand = random()
        if random() > 0.9:
            out_string.append(2.0 * random() - 1.0)
        elif crossover_point <= 0:
            ##if random() > 0.5:
            ##    out_string.append(elem2 + (random() - 0.5) / 4.0)
            ##else:
            out_string.append(elem2)
        else:
            ##if random() > 0.5:
            ##    out_string.append(elem1 + (random() - 0.5) / 4.0)
            ##else:
            out_string.append(elem1)
        crossover_point -= 1
    assert len(out_string) == len(string1)
    return out_string

def pick_parent_per_gene(string1, string2):
    assert len(string1) == len(string2)
    out_string = []
    crossover_point = random() * len(string1)
    for elem1, elem2 in zip(string1, string2):
        rand = random()
        if rand < 0.475:
            out_string.append(elem1)
        elif rand < 0.95:
            out_string.append(elem2)
        else:
            out_string.append(2.0 * random() - 1.0)
    assert len(out_string) == len(string1)
    return out_string

def combine_strings(string1, string2):
    #return pick_one_parent(string1, string2)
    return crossover(string1, string2)
    #return pick_parent_per_gene(string1, string2)

class Dna:
    def __init__(self, dna1=None, dna2=None,
                 num_brain_inputs=None,
                 num_brain_outputs=None):
        if dna1 == None or dna2 == None:
            # Random initialization
            assert num_brain_inputs != None
            assert num_brain_outputs != None
            length = brain.num_weights(num_brain_inputs,
                                       num_brain_outputs) + 7
            self.string = [(2.0 * random()) - 1.0 for i in xrange(length)]
            self.num_brain_inputs = num_brain_inputs
            self.num_brain_outputs = num_brain_outputs
        else:
            # Create string based on dna of both parents
            #self.string = crossover(dna1.string, dna2.string)
            self.string = combine_strings(dna1.string, dna2.string)
            self.num_brain_inputs = dna1.num_brain_inputs
            self.num_brain_outputs = dna1.num_brain_outputs
    
    def mutate(self):
        for i in xrange(len(self.string)):
            if random() > 0.9:
                self.string[i] = 2.0 * random() - 1.0
    
    def convert_to_color(self, stringpart):
        assert len(stringpart) == 3
        color = []
        for val in stringpart:
            scaled = int((val + 0.5) * 256.0)
            if scaled < 0:
                scaled = 0
            elif scaled > 255:
                scaled = 255
            color.append(scaled)
        return (color[0], color[1], color[2])
    
    def get_color(self):
        """ Return body color """
        color = self.convert_to_color(self.string[0:3])
        return (color[0], 0.0, color[2])
        
    def get_eye_color(self):
        """ Return eye color """
        return self.convert_to_color(self.string[3:6])
        
    def get_eye_angle(self):
        #0.25 * math.pi
        value = self.string[6]
        # use sigmoid function to generate dist [0..pi]
        angle = .9 * math.pi * (1.0 / (1.0 + math.exp(value)))
        return angle #sigmoid

    def get_brain(self):
        """ Return initial weigths for a brain """
        weights = self.string[7:]
        b = brain.Brain(self.num_brain_inputs,
                        self.num_brain_outputs, weights)
        return b
